(function() {
  /* 
  Library of manipulator functions.
  */
  var fs, _;
  _ = require('underscore');
  _.mixin(require('underscore.string'));
  fs = require('fs');
  /*
  Pre-manipulators
  
  Pre-manipulators are passed each individual file before it gets merged in to one file.
  
  @param {String} contents The contents of the file
  @param {String} filename
  @return {String} The manipulated contents 
  */
  this.compileCoffeescript = function(contents, filename) {
    if ((filename != null) && (filename.match(/.coffee$/) != null)) {
      return require('coffee-script').compile(contents);
    } else {
      return contents;
    }
  };
  this.compileStylus = function(contents, filename) {
    var css;
    if ((filename != null) && (filename.match(/.styl$/) != null)) {
      css = '';
      require('stylus')(contents).set('filename', filename).use(require('nib')()).render(function(err, out) {
        if (err) {
          throw err;
        }
        return css = out;
      });
      return css;
    } else {
      return contents;
    }
  };
  this.packageJST = function(contents, filename) {
    var escapedFile, ext, tmplDirname;
    tmplDirname = 'templates/';
    ext = _.last(filename.split('.'));
    escapedFile = contents.replace(/\n+/g, '\\n').replace(/"/g, '\\"');
    if ((filename != null) && filename.indexOf(tmplDirname) !== -1) {
      filename = filename.substr(filename.indexOf(tmplDirname) + tmplDirname.length);
    }
    filename = filename.replace('.' + ext, '');
    return "window.JST[\"" + filename + "\"] = JSTCompile(\"" + escapedFile + "\");\n";
  };
  /*
  Post-manipulators
  
  Post-manipulators are passed the merged files.
  
  @param {String} contents The contents of the merged files
  @return {String} The manipulated contents
  */
  this.prependJST = function(compilerFunctionStr) {
    return function(contents) {
      var str;
      str = 'window.JST = {};\n';
      str += "window.JSTCompile = " + compilerFunctionStr + ";\n";
      return str + contents;
    };
  };
  this.ugilfyJS = function(contents) {
    var ast, jsp, pro;
    jsp = require("uglify-js").parser;
    pro = require("uglify-js").uglify;
    ast = jsp.parse(contents);
    ast = pro.ast_mangle(ast);
    ast = pro.ast_squeeze(ast);
    return pro.gen_code(ast);
  };
  this.yuiCssMin = function(contents) {
    return require('../deps/yui_cssmin.js').minify(contents);
  };
  this.embedImages = function(imgDir) {
    return function(contents) {
      var base64, end, file, getIndicesOf, indecies, index, _i, _len;
      getIndicesOf = function(searchStr, str, caseSensitive) {
        var index, indices, searchStrLen, startIndex;
        startIndex = 0;
        searchStrLen = searchStr.length;
        index = void 0;
        indices = [];
        if (!caseSensitive) {
          str = str.toLowerCase();
          searchStr = searchStr.toLowerCase();
        }
        while ((index = str.indexOf(searchStr, startIndex)) > -1) {
          indices.push(index);
          startIndex = index + searchStrLen;
        }
        return indices;
      };
      indecies = getIndicesOf('url(', contents, false);
      for (_i = 0, _len = indecies.length; _i < _len; _i++) {
        index = indecies[_i];
        end = contents.substring(index, contents.length).indexOf(')') + index + 1;
        file = imgDir + contents.substring(index + 4, end - 1);
        base64 = fs.readFileSync(file).toString('base64');
        console.log(index, end);
      }
      return 'foo';
    };
  };
}).call(this);
