(function() {
  var compilePackage, fileUtil, fn, fs, knox, name, path, replaceWildcards, sentry, splitAssetGroup, _, _ref;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fileUtil = require('file');
  _ = require('underscore');
  fs = require('fs');
  path = require('path');
  sentry = require('sentry');
  knox = require('knox');
  _ref = require(__dirname + '/manipulators');
  for (name in _ref) {
    fn = _ref[name];
    this[name] = fn;
  }
  this.package = function(assets, dir, options) {
    var extension, files, keys, packageName, packages, _ref2;
    packages = [];
    for (extension in assets) {
      keys = assets[extension];
      _ref2 = splitAssetGroup(keys).packages;
      for (packageName in _ref2) {
        files = _ref2[packageName];
        packages.push(compilePackage(packageName + '.' + extension, replaceWildcards(files), dir, splitAssetGroup(keys).manipulators, options ? options.env : null));
      }
    }
    return packages;
  };
  this.watch = function(assets, dir) {
    var extension, file, files, keys, packageName, _results;
    _results = [];
    for (extension in assets) {
      keys = assets[extension];
      _results.push((function() {
        var _ref2, _results2;
        _ref2 = splitAssetGroup(keys).packages;
        _results2 = [];
        for (packageName in _ref2) {
          files = _ref2[packageName];
          _results2.push((function() {
            var _i, _len, _results3;
            _results3 = [];
            for (_i = 0, _len = files.length; _i < _len; _i++) {
              file = files[_i];
              _results3.push(sentry.watch(file, function(filename) {
                console.log("Found change in package " + (packageName + '.' + extension) + ", compiling");
                return compilePackage(packageName + '.' + extension, replaceWildcards(files), dir, splitAssetGroup(keys).manipulators);
              }));
            }
            return _results3;
          })());
        }
        return _results2;
      })());
    }
    return _results;
  };
  this.packageToS3 = __bind(function(assets, dir, S3Options, callback) {
    var client, finishPackaging, package, packages, responses, to, _i, _len, _ref2, _results;
    packages = this.package(assets, dir, (_ref2 = S3Options.env) != null ? _ref2 : 'production');
    client = knox.createClient({
      key: S3Options.key,
      secret: S3Options.secret,
      bucket: S3Options.bucket
    });
    responses = [];
    finishPackaging = _.after(packages.length, function(responses) {
      return callback(responses);
    });
    _results = [];
    for (_i = 0, _len = packages.length; _i < _len; _i++) {
      package = packages[_i];
      to = S3Options.dir + '/' + path.basename(package);
      _results.push(client.putFile(package, to, function(err, res) {
        if (err) {
          throw err;
        }
        responses.push(res);
        console.log("Put package " + package + " in S3 bucket '" + S3Options.bucket + "' \n");
        return finishPackaging(responses);
      }));
    }
    return _results;
  }, this);
  splitAssetGroup = function(group) {
    var manipulators, packages;
    packages = _.clone(group);
    manipulators = {
      pre: group.preManipulate,
      post: group.postManipulate
    };
    delete packages['preManipulate'];
    delete packages['postManipulate'];
    return {
      packages: packages,
      manipulators: manipulators
    };
  };
  replaceWildcards = function(files) {
    var file;
    files = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(file.indexOf('/*') !== -1 ? sentry.findWildcards(file) : file);
      }
      return _results;
    })();
    return _.uniq(_.flatten(files));
  };
  compilePackage = function(name, files, dir, manipulators, env) {
    var concatFileStr, file, fileStrs, i, manipulator, _i, _j, _len, _len2, _ref2, _ref3, _ref4, _ref5, _ref6;
    env = (_ref2 = env != null ? env : process.env.NODE_ENV) != null ? _ref2 : 'development';
    fileStrs = (function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(fs.readFileSync(file).toString());
      }
      return _results;
    })();
    for (i in files) {
      file = files[i];
      if ((manipulators != null) && (manipulators.pre != null) && ((manipulators.pre[env] != null) || (manipulators.pre['*'] != null))) {
        _ref4 = (_ref3 = manipulators.pre[env]) != null ? _ref3 : manipulators.pre['*'];
        for (_i = 0, _len = _ref4.length; _i < _len; _i++) {
          manipulator = _ref4[_i];
          fileStrs[i] = manipulator(fileStrs[i], file);
        }
      }
    }
    concatFileStr = ((function() {
      var _j, _len2, _results;
      _results = [];
      for (_j = 0, _len2 = fileStrs.length; _j < _len2; _j++) {
        file = fileStrs[_j];
        _results.push(file);
      }
      return _results;
    })()).join('\n');
    if ((manipulators != null) && (manipulators.post != null) && ((manipulators.post[env] != null) || (manipulators.post['*'] != null))) {
      _ref6 = (_ref5 = manipulators.post[env]) != null ? _ref5 : manipulators.post['*'];
      for (_j = 0, _len2 = _ref6.length; _j < _len2; _j++) {
        manipulator = _ref6[_j];
        concatFileStr = manipulator(concatFileStr);
      }
    }
    fs.writeFileSync("" + dir + "/" + name, concatFileStr);
    return "" + dir + "/" + name;
  };
}).call(this);
